
/*

     C primitives for writing a qcd data file generated by my occam program.

*/

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/file.h>
#include "spnhdr.h"
#include "strmhdr.h"

/* microseconds */
/*
#define DELAY 200000
*/

extern float get_float () ;
extern int   get_int   () ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

static void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "dsource: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

DATA *data_dopen_write (fildes)
int fildes ;
{
  DATA *result ;

  result = (DATA *) malloc (sizeof (DATA)) ;

  result->desc = fildes ;
  result->save_full = 1 ;

  return result ;
}

DATA *data_open_write (file)
char *file ;
{
  int fildes ;

  if (file == NULL)
    fildes = 0 ;
  else
  {
    char err_mess [128] ;

    fildes = open (file, O_CREAT | O_TRUNC | O_WRONLY, 0) ;

    strcpy (err_mess, "Failed to open \"") ;
    strcat (err_mess, file) ;
    strcat (err_mess, "\"") ;
    sys_assert (fildes >= 0, err_mess) ;
  }

  return data_dopen_write (fildes) ;
}

void data_close_write (data)
DATA *data ;
{
  int result ;

  close (data->desc) ;
  free  (data) ;
}

static void write_buffer (data, len_to_write)
DATA *data ;
int len_to_write ;
{
  int result ;

  result   = write (data->desc, data->buf, len_to_write) ;
  sys_assert (result > 0, "Write failed") ;
  bzero (data->buf, BUFFER_SIZE) ;
}

void write_header (data)
DATA *data ;
{
  bzero (data->buf, BLOCK_SIZE) ;
  write_buffer (data, BLOCK_SIZE) ;
}

void write_coeffs (data, num_coeffs, coeffs)
DATA *data ;
int num_coeffs ;
float coeffs [] ;
{
  int *buf ;

  buf = (int *) data->buf ;
  bzero (buf, BLOCK_SIZE) ;
  {
    int i ;

    buf [0] = put_int (num_coeffs) ;
    for (i = 0; i < num_coeffs; i++)
      buf [i + 1] = put_float (coeffs [i]) ;
  }

  write_buffer (data, BLOCK_SIZE) ;
}

void write_params (data, num_params, params)
DATA *data ;
int num_params ;
int params [] ;
{
  int *buf ;

  bzero (buf, BLOCK_SIZE) ;
  buf = (int *) data->buf ;
  {
    int i ;

    buf [0] = put_int (num_params) ;
    for (i = 0; i < num_params; i++)
      buf [i + 1] = put_int (params [i]) ;
  }

  write_buffer (data, BLOCK_SIZE) ;
}

void write_field_begin (data)
DATA *data ;
{
  data->site_size        = sizeof (float) * N_SPIN * N_COL * 2 ;
  data->sites_per_buffer = BUFFER_SIZE / data->site_size ;
  data->buf_ptr          = 0 ;
  bzero (data->buf, BUFFER_SIZE) ;
}

void write_field_record (data, field_record)
DATA *data ;
char *field_record ;
{
  if (data->buf_ptr == data->sites_per_buffer)
  {
    if (data->save_full)
      write_buffer (data, BUFFER_SIZE) ;
    else
      write_buffer (data, data->site_size * data->sites_per_buffer) ;
    data->buf_ptr = 0 ;
  }
  memcpy (data->buf + (data->site_size * data->buf_ptr),
	  field_record, data->site_size) ;
  (data->buf_ptr)++ ;
}

void write_field_end (data)
DATA *data ;
{
  if (data->buf_ptr > 0)
  {
    if (data->save_full)
      write_buffer (data, BUFFER_SIZE) ;
    else
      write_buffer (data, data->site_size * data->buf_ptr) ;
  }
}

